unit UC_RowGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
type
  TRowGrid = Class; //Forward

  {Stockage des largeurs de colonnes}
  TColInfos = Record
    Name    : String;  //Nom de la colonne
    Percent : Single;  //Largeur de la colone en %
    Right   : Integer; //Position bordure droite
    UserObj : TObject; //Donne utilisateur pour TStrings
  End;
  TColsInfos = Array[0..MaxLongInt div SizeOf(TColInfos) -1] of TColInfos;
  PColsInfos = ^TColsInfos;

  {Stockage des donnes}
  TCell = Record   //Donnes d'une cellule
    Str : String;     //La chaine
    Dat : Integer;    //La donne associe
  End;
    //Donnes d'une ligne
  TRowFlags=Integer;
  TRow = Packed Record
    Flags:TRowFlags;
    Cells:Array[0..MaxLongInt div SizeOf(TColInfos) -1] of TCell;
  End;
  PRow = ^TRow;  //Pointeur de fPTabRows
  PProw = ^PRow; //Pointeur de fPFilter
    //Table des lignes
  TTabRows = Array[0..MaxLongInt div SizeOf(PRow) -1] of PRow;
  PTabRows = ^TTabRows;
    //Sauvegarde de la row courante
  TRowPtrSave = Record
    Ptr:PRow;
    Idx:Integer;
  End;

  {Dessin des colonnes}
  TVertAlignment = (taTopJustify, taBottomJustify, taMiddle);
  TDrawCellInfo = record
    Row,Col:Integer;
    Text:String;
    Brush: TBrush;
    Font: TFont;
    AlignmentHorz: TAlignment;
    AlignmentVert: TVertAlignment;
  end;
  TGetFormatDrawCellEvent = Procedure(Sender:TObject; var Infos:TDrawCellInfo) of object;

  TColorsId = (ciEvenRow, ciOddRow, ciHeaderRow, ciFocusedRow, ciFocusedCell,
               ciEditor, ciSelected);
  TRowGridColors = Array[TColorsId] of TColor;

  TIndexEvent = Procedure(Sender:TObject; Index:Integer) of object;
  TCoordEvent = Procedure(Sender:TObject; Row,Col:Integer) of object;
  TCoordCheck = Function(Sender:TObject; Row,Col:Integer):Boolean of object;

  TRowCheck   = Function(Sender:TObject; Row:Integer):Boolean of object;
  TSelectCellEvent = Procedure (Sender:TObject; NewRow,NewCol:Integer; var CanSelect:Boolean) of object;

  {Entre et sortie d'dition}
  TEditBaseClass = TWinControl;
  TCreateEditor =Function(Sender:TObject; Col:Integer):TEditBaseClass of Object;
  TEnterEditProc=Procedure(Sender:TObject; Row,Col:Integer; Var Text:String) of object;
  TExitEditProc=Procedure(Sender:TObject; Row,Col:Integer; Var NewText:String) of object;

  {Tri des lignes}
  TSortElement = Record
    Key1:Pointer; //Pointeur vers donne  trier, initialis par GetKeyFunc
    Key2:Pointer;
    Index:Integer; //Index original
  End;
  TSortDirection  = (sdAscending, sdDescending);
  TSortGetKeyFunc = Procedure(ACol:Integer; VAR Item:TSortElement) of object;
  TSortCompFunc   = Function(Const Item1,Item2:TSortElement):Integer of object;
  TSortInsSepFunc = Function(Const Item1,Item2:TSortElement):Boolean of object;

  {Copier coller}
  TPasteFunc = Function(Var RowToPaste:TRow):Boolean of object;

  //------------------------------
  TRowGrid = class(TCustomControl)
  {Aspect, taille de la fentre}
  Private
    fBorderStyle : TBorderStyle; //Type de bordure de fentre
    fRowHeight   : Integer;      //Hauteur des lignes
    fColors      : TRowGridColors; //Couleurs de fond
    Procedure SetBorderStyle(Style:TBorderStyle);
    Procedure SetCtl3D(NewState:Boolean);
    Function GetColor(Index:TColorsId):TColor;
    Procedure SetColor(Index:TColorsId; NewColor:TColor);
    Procedure SetRowHeight(NewHeight:Integer);
    Procedure WMSize(var Msg:TWMSize); Message WM_SIZE;
  Protected
    Procedure SetParent(AParent:TWinControl); Override;
  Public
    Constructor Create(AOwner:TComponent); Override;
    Destructor Destroy; Override;
    Procedure CreateParams(var Params:TCreateParams); Override;
    Procedure SetColors(NewColors:TRowGridColors);
    Property Colors[Index:TColorsId]:TColor read GetColor write SetColor;
  Published
    Property BorderStyle: TBorderStyle read fBorderStyle write SetBorderStyle default bsSingle;
    Property Align;
    Property Ctl3D write SetCtl3D;
    Property DefaultRowHeight:Integer read fRowHeight write SetRowHeight default 18;
    Property Enabled;
    property ParentCtl3D;
    Property ParentShowHint;
    property PopupMenu;
    Property ShowHint;
    Property TabOrder;
    Property TabStop default True;
    Property Visible;

  {Gestion de la zone visible}
  Private
    fTopRow:Integer;            //Numro de la premire ligne visible
    fVisibleRowCount:Integer;   //Nombre de lignes visibles
    Procedure MajScrollBar;
    Procedure WMVScroll(var Msg:TWMScroll); Message WM_VSCROLL;
    Procedure SetTopRow(NewTopRow:Integer);
  Public
    Property TopRow:Integer read fTopRow write SetTopRow;
    Property VisibleRowCount:Integer read fVisibleRowCount;

  {Dessin  l'cran}
  Private
    fOnGetCellFormat: TGetFormatDrawCellEvent;
    Function CellRect(ARow,ACol:Integer): TRect;
    Procedure WMEraseBkGnd(var Msg:TMessage); Message WM_ERASEBKGND;
    Procedure PaintAllRows;
    Procedure PaintZone(StartRow,EndRow:Integer); //Affichage de StartRow  EndRow
    Procedure PaintCell(ARow,ACol:Integer); //Raffichage d'une seule cellule
  Protected
    Procedure DrawHeaderRow; Virtual; //Affichage de la ligne d'entte
    Procedure DrawCell(ARow,ACol:Integer; ARect:TRect); Virtual;
  Public
    Procedure Paint; Override; {Reaffichage de tout l'cran}
    Procedure PaintRow(Index:Integer);
  Published
    Property OnGetCellFormat:TGetFormatDrawCellEvent read fOnGetCellFormat write fOnGetCellFormat;
    Property Font;
    Property ParentFont;

  {Gestion des colonnes}
  Private
    fPColsInfos: PColsInfos;    //Largeurs des colonnes
    fColCount : Integer;        //Nombre de colonnes par ligne
    fColNames : TStrings;       //Noms des colonnes
    fColSizing : Boolean;       //Redimentionnement colonne en cours
    fColSizeLine : Integer;     //Position de la ligne de dimentionnement de colonne
    fPenMode : TPenMode;        //Sauve PenMode pendant redimentionement
    fColCountChange:Boolean;
    Procedure SetColCountNoPaint(NewCount:Integer);
    Procedure SetColCountPaint(NewCount:Integer);
    Procedure UpdateColsWidths;
    Function  GetColWidth(Index:Integer):Integer;
    Function GetColName(Index:Integer):String;
    Procedure SetColName(Index:Integer; Const Name:String);
    Procedure SetColNames(Names:TStrings);
  Protected
    Procedure ColDefsChange;
  Public
    Property ColName[Index:Integer]:String read GetColName write SetColName;
    Property ColWidths[Index:Integer]:Integer read GetColWidth;
    Procedure DefColWidths(Const Widths:Array of Integer);
    Procedure DefColNames(Const Names:Array of String);
    Property ColCount:Integer read fColCount;// write SetColCountPaint default 1;
  Published
    Property ColNames:TStrings read fColNames write SetColNames;

  {Filtre}
  Private
    fFilterList:TList; //Sauve liste de numros de lignes utiliss
    fOnFilter:TCoordCheck;
  Public
    Procedure SetFilter(NumCol:Integer);
  Published
    Property OnFilter:TCoordCheck read fOnFilter write fOnFilter;

  {Gestion des donnes}
  Private
    fPTabRows  : PTabRows;   //Pointeur vers la table des lignes
    fRowCount  : Integer;    //Nombre de lignes utilises
    fRowsAlloc : Integer;    //Nombre de lignes alloues dans la table
    fRowPtr    : PRow;       //Ligne en cours d'accs par utilisateur
    fRowPtrNum : Integer;    //N de la ligne en cours d'accs

    fUpdateCount : Integer;      //Compteur de BeginUpdate/EndUpdate
    fReadOnly    : Boolean;      //Empche les modif par utilisateur (pas par programme)
    fModified    : Boolean;      //Indicateur de modifications
    fOnModif     : TNotifyEvent; //Appele quand fModified change
    fOnChange    : TNotifyEvent; //Appele sur modifs

    fOnInitNewRow:TIndexEvent;  // Initialisation de nouvelle ligne
    fOnInsertRow: TIndexEvent;  // Insertion de ligne
    fOnDeleteRow: TIndexEvent;  // Suppression de ligne
    fOnDeleteRowQuery: TRowCheck; //Appele avant supprression de ligne si non vide
    fOnRowEmptyCheck : TRowCheck;  //Appele pour dterminer si ligne vide
    Procedure SaveRowPtr(VAR Place:TRowPtrSave);
    Procedure LoadRowPtr(CONST Place:TRowPtrSave);

    Function GetRowCount:Integer;
    Procedure SetRowCountNoPaint(NewCount:Integer);
    Procedure SetRowCountPaint(NewCount:Integer);
    Procedure AddRowVide;

    Function GetRowFlags:TRowFlags;
    Procedure SetRowFlags(NewFlags:TRowFlags);
    Function GetRowCellStr(Col:Integer):String;     //Accs par ligne
    Procedure SetRowCellStr(Col:Integer; S:String);
    Function GetRowCellDat(Col:Integer):Integer;
    Procedure SetRowCellDat(Col,Data:Integer);

    Procedure InsertRowNoPaint(Index:Integer; PtrNewRow:PRow);
    Procedure MoveRowNoPaint(FromIndex,ToIndex:Integer);
  Protected
    Procedure SignaleModif; Virtual;  //Pour appeller OnChange si autoris
  Public
    Procedure BeginUpdate;
    Procedure EndUpdate;
    Property Modified:Boolean read fModified Write fModified default False;

    Procedure InsertRow(Index:Integer); Virtual; {Insre avant la ligne N}
    Procedure DeleteRow(Index:Integer); Virtual; {Supprime la ligne N}
    Procedure MoveRow(FromIndex,ToIndex:Integer);
    Function IsRowEmpty(Row:Integer):Boolean;

    Procedure SelectRow(RowNum:Integer);
    Property RowFlags:TRowFlags read GetRowFlags write SetRowFlags;
    Property RowTexts[Col:Integer]:String  read GetRowCellStr write SetRowCellStr;
    Property RowDatas[Col:Integer]:Integer read GetRowCellDat write SetRowCellDat;
  Published
    Property ReadOnly:Boolean read fReadOnly write fReadOnly;
    Property RowCount:Integer read GetRowCount write SetRowCountPaint default 0;
    Property OnChange:TNotifyEvent read fOnChange write fOnChange;
    Property OnModifiedToggle:TNotifyEvent read fOnModif write fOnModif;
    Property OnRowEmptyCheck:TRowCheck read fonRowEmptyCheck write fonRowEmptyCheck;
    Property OnInitNewRow:TIndexEvent read fOnInitNewRow write fOnInitNewRow;
    property OnInsertRow:TIndexEvent read fOnInsertRow write fOnInsertRow;
    Property OnDeleteRowQuery:TRowCheck read fOnDeleteRowQuery write fOnDeleteRowQuery;
    property OnDeleteRow:TIndexEvent read fOnDeleteRow write fOnDeleteRow;

  {Focus et Gestion de la cellule et ligne slectionne}
  Private
    fCol : Integer; //Colone slectionne
    fRow : Integer; //Ligne slectionne
    fOnSelectCell:TSelectCellEvent;
    fOnColChange:TIndexEvent;
    fOnRowChange:TIndexEvent;
    Procedure WMSetFocus (var Msg:TWMSetFocus);  message WM_SETFOCUS;
    Procedure WMKillFocus(var Msg:TWMKillFocus); message WM_KILLFOCUS;
    Procedure SetActiveRow(NewRow:Integer);
    Procedure SetActiveCol(NewCol:Integer);
    Function CanSelect(ARow,ACol:Integer):Boolean;
  Public
    Property Col:Integer read fCol write SetActiveCol;
    Property Row:Integer read fRow write SetActiveRow;
  Published
    Property OnEnter;
    Property OnExit;
    Property OnSelectCell:TSelectCellEvent read fOnSelectCell write fOnSelectCell;
    Property OnColChange:TIndexEvent read fOnColChange write fOnColChange;
    Property OnRowChange:TIndexEvent read fOnRowChange write fOnRowChange;

  {Gestion du mode dition}
  Private
    fOnCreateEditor:TCreateEditor;
    fEdit : TEditBaseClass;
    fOnEnterEdit:TEnterEditProc;
    fOnValidEdit:TExitEditProc;
    fOnExitEdit :TCoordEvent;
    Procedure SetEditplace;
  Protected
    Procedure EnterEdit;
  Public
    Property Editor:TWinControl read fEdit;
    Function Editing:Boolean;
  Published
    Property OnCreateEdit:TCreateEditor read fOnCreateEditor write fOnCreateEditor;
    Property OnEnterEdit:TEnterEditProc read fOnEnterEdit write fOnEnterEdit;
    Property OnValidEdit:TExitEditProc read fOnValidEdit write fOnValidEdit;
    Property OnExitEdit :TCoordEvent read fOnExitEdit  write fOnExitEdit;

  {Souris}
  Private
    fMouseDownCol : Integer;    //Colonne du MouseDown pour dtection MouseUp ds mme col
    procedure CMDesignHitTest(var Msg:TCMDesignHitTest); Message CM_DESIGNHITTEST;
  Protected
    Procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); Override;
    Procedure MouseMove(Shift:TShiftState; X,Y:Integer); Override;
    Procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); Override;
  Public
    Procedure MouseToCell(X,Y:Integer; var ARow,ACol:Integer);
  Published
    Property Cursor default crIBeam;
    Property DragCursor;
    Property DragMode;

    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
    Property OnClick;
    Property OnDblClick;

    Property OnStartDrag;
    Property OnEndDrag;
    Property OnDragOver;
    Property OnDragDrop;

  {Clavier}
  Private
    Procedure WMGetDlgCode(var Msg:TWMGetDlgCode); message WM_GETDLGCODE;
  Protected
    Procedure KeyDown(var Key:Word; Shift:TShiftState); Override;
    Procedure KeyPress(var Key:Char); Override;
  Published
    Property OnKeyDown;
    Property OnKeyPress;
    Property OnKeyUp;

  {Tris}
  Private
    fSortEnable:Boolean;
    fSortCol:Integer;
    fSortDirection: TSortDirection;

    fOnSortStart:TIndexEvent;
    fOnSortGetKey:TSortGetKeyFunc;
    fOnSortComp:TSortCompFunc;
    fOnSortEnd:TIndexEvent;
    fOnSortInsSep:TSortInsSepFunc;
    Procedure SetSortEnable(Etat:Boolean);
  Public
    Function SortInteger(Const Item1,Item2:TSortElement):Integer;
    Function SortStrings(Const Item1,Item2:TSortElement):Integer;

    Procedure SortColumn(ACol:Integer);
    Property SortCol:Integer read fSortCol;
  Published
    Property SortEnable:Boolean read fSortEnable write SetSortEnable;
    Property OnSortStart:TIndexEvent read fOnSortStart write fOnSortStart;
    Property OnSortGetKey:TSortGetKeyFunc read fOnSortGetKey write fOnSortGetKey;
    Property OnSortCompare:TSortCompFunc read fOnSortComp write fOnSortComp;
    Property OnSortEnd:TIndexEvent read fOnSortEnd write fOnSortEnd;
    Property OnSortInsSep:TSortInsSepFunc read fOnSortInsSep write fOnSortInsSep;

  {Copier/Couper/Coller}
  Private
    fClipboardId:Integer;  //ID pour accepter reconnatre grille dans clipboard
    fOnPaste:TPasteFunc;
    fSelCount:Integer;     //Nombre de lignes slectionnes
  Public
    Property SelCount:Integer read fSelCount;
    Procedure CopyToClipboard;
    Procedure CutToClipBoard;
    Procedure PasteFromClipboard;
    Function CanPaste:Boolean;
  Published
    Property ClipBoardId:Integer read fClipboardId write fClipboardId default 0;
    Property OnPasteRow:TPasteFunc read fOnPaste write fOnPaste;

  {Chargement/Sauvegarde proprits}
  Private
    procedure ReadColWidths(Reader: TReader); //Pour sauvegarde  conception
    procedure WriteColWidths(Writer: TWriter);
  Protected
    procedure DefineProperties(Filer: TFiler); override; //Pour sauvegarde  conception
  end;

Const
  //Dfinitions de TRowFlags :
  rfSel = $0001;  //Ligne slectionne
Procedure Register;

////////////////////////////////////////////////////////////////////////////////

implementation

Var
  CF_TRowGridRow:Integer; //Pour couper/copier/coller


procedure Register;
begin
  RegisterComponents('Perso', [TRowGrid]);
end;

//--------------------------------------------------------------------------
// Quelques outils hors objets
//--------------------------------------------------------------------------

Function Min(X, Y:Integer):Integer;
Asm
  cmp eax,edx
  jl  @Fin
  mov eax,edx
@Fin:
End;

Function Max(X, Y:Integer):Integer;
Asm
  cmp eax,edx
  jg  @Fin
  mov eax,edx
@Fin:
End;

{Force X  rester dans la plage [0,Max] avec Max>=0}
Procedure Limite0Max(Var X:Integer; Max:Integer);
Begin
  If X>Max then X:=Max;
  If X<0 then X:=0;
End;

//--------------------------------------------------------------------------
// TRowGridEditor : diteur de la grille
//--------------------------------------------------------------------------

Type
  TRowGridEdit=Class(TEdit)
  Private
    fRowGrid:TRowGrid;
    fStartText:String;
    Procedure DoEnter; Override;
    Procedure DoExit; Override;
    Procedure KeyPress(var Key:Char); Override;
  Public
    Constructor Create(AOwner: TComponent); override;
    Property Color;
  End;

Constructor TRowGridEdit.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  fRowGrid:=TRowGrid(AOwner);
  Color:=fRowGrid.fColors[ciEditor];
  Autoselect:=False;
End;

Procedure TRowGridEdit.DoEnter;
Begin
  With fRowGrid do
  Begin
    fStartText:=fPTabRows^[fRow]^.Cells[fCol].Str;
    If Assigned(fOnEnterEdit) then
      fOnEnterEdit(fRowGrid, fRow, fCol, fStartText);
  End;
  Text:=fStartText;
  SetFocus;
End;

Procedure TRowGridEdit.DoExit;
Var
  NewText:String;
  Sauve:TRowPtrSave;
Begin
  NewText:=Self.Text;
  With fRowGrid do
  Begin
    If NewText<>fStartText then
    Begin
      SaveRowPtr(Sauve);
      If Assigned(fOnValidEdit) then
        OnValidEdit(fRowGrid, fRow,fCol, NewText);
      If NewText<>fStartText then
      Begin
        fPTabRows^[fRow]^.Cells[fCol].Str:=NewText;
        SignaleModif;

        //Ajoute une ligne si dernire n'est plus vide
        AddRowVide;
      End;
      LoadRowPtr(Sauve);
    End;
    If Assigned(fOnExitEdit) then
      OnExitEdit(fRowGrid, fRow,fCol);
  End;
  Hide;
  Inherited
End;

Procedure TRowGridEdit.KeyPress(var Key:Char);
Begin
  Case Key of
    #27:  //ESC
    Begin
      Text:=fStartText;  //Remet texte initial
      fRowGrid.SetFocus; //et abandonne dition
      Key:=#0;
    End;
    #$0D: //CR
    Begin
      fRowGrid.SetFocus; //Quitte dition avec prise en compte modifs
      Key:=#0;
    End;
    else
      Inherited;
  End;
End;

//--------------------------------------------------------------------------
// TColNameStrings : TStrings pour accs aux noms de colonnes
//--------------------------------------------------------------------------

Type
  TColNameStrings = class(TStrings)
  private
    fRowGrid: TRowGrid;
    fFirstColLoaded:Boolean; //Initialis  false
    fUpdating:Boolean;
  protected
    function Get(Index: Integer): string; override;
    function GetCount: Integer; override;
    function GetObject(Index: Integer): TObject; override;
    procedure Put(Index: Integer; const S: string); override;
    procedure PutObject(Index: Integer; AObject: TObject); override;
    procedure SetUpdateState(Updating: Boolean); override;
  public
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Insert(Index: Integer; const S: string); override;
    procedure Assign(Source: TPersistent); override;
  end;

  ERowGridColNamesError=Class(Exception);

procedure RowGridError;
begin
  raise ERowGridColNamesError.Create('RowGrid.ColNames[] error');
end;

function TColNameStrings.GetCount:Integer;
begin
  If fFirstColLoaded
    then Result:=fRowGrid.fColCount
    else Result:=0;
end;

function TColNameStrings.Get(Index:Integer):string;
begin
  If (Index<0) or (Not fFirstColLoaded) or (Index>=fRowGrid.fColCount) then
  Begin
    Result:='';
    RowGridError;
  End
  else
    Result:=fRowGrid.fPColsInfos^[Index].Name
end;

function TColNameStrings.GetObject(Index:Integer):TObject;
Begin
  If (Index<0) or (Not fFirstColLoaded) or (Index>=fRowGrid.fColCount) then
  Begin
    Result:=Nil;
    RowGridError;
  End
  else
    Result:=fRowGrid.fPColsInfos^[Index].UserObj;
End;

Procedure TColNameStrings.Put(Index:Integer; const S:string);
Begin
  If Index>=fRowGrid.fColCount then
  Begin
    fRowGrid.SetColCountNoPaint(Index+1);
    fRowGrid.fColCountChange:=True;
  End;
  fRowGrid.fPColsInfos^[Index].Name:=S;
  If not fUpdating then
    fRowGrid.ColDefsChange;
End;

procedure TColNameStrings.PutObject(Index: Integer; AObject: TObject);
Begin
  If Index>=fRowGrid.fColCount then
  Begin
    fRowGrid.SetColCountNoPaint(Index+1);
    fRowGrid.fColCountChange:=True;
  End;
  fRowGrid.fPColsInfos^[Index].UserObj:=AObject;
End;

procedure TColNameStrings.Clear;
begin
  If fFirstColLoaded then
  Begin
    fFirstColLoaded:=False;
    fRowGrid.SetColCountNoPaint(1);
    With fRowGrid.fPColsinfos^[0] do
    Begin
      Name   :='';
      UserObj:=Nil;
    End;
    If not fUpdating then
      fRowGrid.ColDefsChange;
  End;
end;

procedure TColNameStrings.Insert(Index:Integer; const S: string);
begin
  With fRowGrid do
  Begin
    If fFirstColLoaded
      then SetColCountNoPaint(fColCount+1) //+1:pour nouvelle col
      else fFirstColLoaded:=True;

    System.Move(fPColsInfos^[Index], fPColsInfos[Index+1], (fColCount-1-Index)*SizeOf(TColInfos));
    Pointer(fPColsInfos^[Index].Name):=Nil; //Empche dcomptage chaine
    fPColsInfos^[Index].UserObj:=Nil;
    fPColsInfos^[Index].Name:=S;
    If not fUpdating then
      fRowGrid.ColDefsChange;
  End;
end;

procedure TColNameStrings.Delete(Index:Integer);
begin
  With fRowGrid do
    If (Index<0) or (Index>=fRowGrid.fColCount) then
      RowGridError
    else
    Begin
      fPColsInfos^[Index].Name:=''; //libre la chaine
      System.Move(fPColsInfos^[Index+1], fPColsInfos[Index], (fColCount-1-Index)*SizeOf(TColInfos));
      Pointer(fPColsInfos^[fColCount-1].Name):=Nil; //Empche dcomptage chaine
      fRowGrid.SetColCountNoPaint(fColCount-1);
      If not fUpdating then
        fRowGrid.ColDefsChange;
    End;
end;

procedure TColNameStrings.Assign(Source: TPersistent);
Var
  SrcStr:TStrings absolute Source;
  I:Integer;
Begin
  If Source is TStrings then
    With fRowGrid do
      If SrcStr.Count=0 then
        Clear
      else
      Begin
        SetColCountNoPaint(SrcStr.Count);
        fFirstColLoaded:=True;
        For I:=0 to SrcStr.Count-1 do
        With fPColsinfos^[I] do
        Begin
          Name   :=SrcStr.Strings[I];
          UserObj:=SrcStr.Objects[I];
        End;
      End
  else
    Inherited;
end;

procedure TColNameStrings.SetUpdateState(Updating: Boolean);
begin
  fUpdating:=Updating;
  If not Updating then
    fRowGrid.ColDefsChange;
end;

//--------------------------------------------------------------------------
// Cration / Destruction / Taille / Aspect
//--------------------------------------------------------------------------

Const
  DefColors : TRowGridColors =
    ({even}clWindow, {Odd}$00FFF0F0, {ciHeaderRow}clBtnFace,
     {ciFocusedRow}$00FFC0C0, {ciFocusedCell}$00FF8080,
     {Editor}$00C0C0FF,{Selected}$00FFFFC0);

Constructor TRowGrid.Create(AOwner:TComponent);
Begin
  Inherited;
//  if NewStyleControls
//    then ControlStyle:=ControlStyle + [csOpaque]
//    else ControlStyle:=ControlStyle + [csOpaque,csFramed];
  fBorderStyle :=bsSingle; //Type de bordure de fentre
  fRowHeight   :=18;       //Hauteur des lignes
//fPColWidths  :Nil; implicite
  fColors      := DefColors; //Couleurs de fond

//fMouseDown :=False; implicite
//fDbleClick :false;  implicite

//fColCount:=0;       implicite
  fColNames:=TColNameStrings.Create;
  TColNameStrings(fColNames).fRowGrid:=Self;
//fRowsAlloc:=0;      implicite
//fRowCount:=0;       implicite
  SetColCountNoPaint(1);
  SetRowCountNoPaint(0);     //Force cration dernire ligne -> Met fColCount=1

//fTopRow:=0;         implicite
//fCol :=0; implicite
//fRow :=0; implicite

//fUpdateCount:=0;    implicite
//fModified :=False;  implicite

//fFilterList:=Nil;   implicite

//fSortEnable:=False; implicite
//fSortCol:=0;        implicite
//fSortDirection:sdAscending implicite

  Width:=320;
  Height:=120;
End;

Destructor TRowGrid.Destroy;
Var C,R:Integer;
Begin
  fFilterList.free; //Supprime la liste de filtre/tri
  fColNames.Free;   //Dtruit liste si existante

  FreeMem(fPColsInfos);       //Libre largeurs de colonnes
  For R:=0 to fRowCount-1 do
  Begin
    For C:=0 to fColCount-1 do    //Libre chaines de la ligne
      fPTabRows^[R]^.Cells[C].Str:='';
    FreeMem(fPTabRows^[R]);       //Libre ligne
  End;
  FreeMem(fPTabRows);         //Libre table des lignes
  Inherited;
End;

Procedure TRowGrid.CreateParams(var Params:TCreateParams);
Const
  BorderStyles: Array[TBorderStyle] of DWORD=(0,WS_BORDER);
Begin
  Inherited CreateParams(Params);
  With Params do
  Begin
    Style:=Style or WS_TABSTOP  //Peut recevoir focus
                 or WS_VSCROLL  //Ascenseur vertical
                 or BorderStyles[fBorderStyle]
                 or WS_CLIPCHILDREN
                 or WS_EX_TRANSPARENT;
    WindowClass.style := CS_DBLCLKS;
    If FBorderStyle = bsSingle then
      If NewStyleControls and Ctl3D then
      Begin
        Style:=Style and not WS_BORDER;
        ExStyle:=ExStyle or WS_EX_CLIENTEDGE;
      End
      else
        Style := Style or WS_BORDER;
  End;
End;

Procedure TRowGrid.SetBorderStyle(Style:TBorderStyle);
Begin
  If fBorderStyle<>Style then
  Begin
    fBorderStyle:=Style;
    RecreateWnd;
  End;
End;

Procedure TRowGrid.SetCtl3D(NewState:Boolean);
Begin
  If Ctl3D<>NewState then
  Begin
    Inherited Ctl3D:=NewState;
    RecreateWnd;
  End;
End;

Function TRowGrid.GetColor(Index:TColorsId):TColor;
Begin
  Result:=fColors[Index];
End;

Procedure TRowGrid.SetColor(Index:TColorsId; NewColor:TColor);
Begin
  If fColors[Index]<>NewColor then
  Begin
    fColors[Index]:=NewColor;
    If Index=ciHeaderRow
      then DrawHeaderRow
      else PaintAllRows;       //Repeint tout avec nouvelle couleur
  End;
End;

Procedure TRowGrid.SetColors(NewColors:TRowGridColors);
Begin
  If not CompareMem(@fColors, @NewColors, SizeOf(TRowGridColors)) then
  Begin
    fColors:=NewColors;
    PaintAllRows;
  End;
End;

Procedure TRowGrid.SetParent(AParent:TWinControl);
Var H:Integer;
Begin
  Inherited;
  If Assigned(AParent) then
  Begin
    H:=Canvas.TextHeight('Hy');
    If H>fRowHeight-4 then
      fRowHeight:=H+4;
  End;
End;

Procedure TRowGrid.SetRowHeight(NewHeight:Integer);
Var
  Msg:TWMSize;
Begin
  If NewHeight<>fRowHeight then
  Begin
    fRowHeight:=NewHeight;
    WMSize(Msg); //Recalcule nombre de lignes, scrollbar et position diteur
    Paint; //Repeint tout y compris entte
  End;
End;

procedure TRowGrid.WMSize(var Msg:TWMSize);
begin
  fVisibleRowCount:=ClientHeight div fRowHeight -1; //-1 car 1 ligne fixe
  If fTopRow>=fRowCount-fVisibleRowCount then
    fTopRow:=Max(0,fRowCount-fVisibleRowCount);

  MajScrollBar;
  UpdateColsWidths;
  If Editing then
    SetEditplace; //En train d'diter valeur -> recentre diteur sur sa case
  Paint;
  Msg.Result:=0;
end;

//--------------------------------------------------------------------------
// Gestion de la zone visible
//--------------------------------------------------------------------------

{Traite demande de dfilement par dplacement de l'ascenseur}
Procedure TRowGrid.WMVScroll(var Msg:TWMScroll);
Var PremLig:Integer;
Begin
  PremLig:=fTopRow;
  Case Msg.ScrollCode of
    SB_BOTTOM:      PremLig:=fRowCount-fVisibleRowCount; // En buts basse
    SB_ENDSCROLL:   Exit;                          // Pas de dplacement !
    SB_LINEDOWN:    Inc(PremLig);                  // 1 ligne vers haut
    SB_LINEUP:      Dec(PremLig);                  // 1 ligne vers bas
    SB_PAGEDOWN:    Inc(PremLig,fVisibleRowCount); // 1 page vers haut
    SB_PAGEUP:      Dec(PremLig,fVisibleRowCount); // 1 page vers bas
    SB_THUMBPOSITION,                              // Suit dplacement
    SB_THUMBTRACK:  PremLig:=Word(Msg.pos);        // Limite = 65536 Lignes
    SB_TOP:         PremLig:=0;                    // En bute haute
  End;
  SetTopRow(PremLig);
  If Editing then
    SetEditPlace;
End;

{Positionne scrollBar conformment  fentre}
Procedure TRowGrid.MajScrollBar;
Var
  ScrollInfo: TScrollInfo;
Begin
  ScrollInfo.cbSize    := SizeOf(ScrollInfo);
  ScrollInfo.fMask     := SIF_ALL {or SIF_DISABLENOSCROLL};

  // Ajuste barre verticale
  ScrollInfo.nMin  := 0;
  ScrollInfo.nMax  := fRowCount-1;
  ScrollInfo.nPage := fVisibleRowCount;
  ScrollInfo.nPos  := fTopRow;
  SetScrollInfo(Handle,SB_VERT,ScrollInfo,True);
End;

Procedure TRowGrid.SetTopRow(NewTopRow:Integer);
Begin
  If fTopRow<>NewTopRow then
  Begin
    fTopRow:=NewTopRow;
    Limite0Max(fTopRow, fRowCount-fVisibleRowCount);
    MajScrollBar;
    PaintAllRows;
  End;
End;

//--------------------------------------------------------------------------
// Affichage
//--------------------------------------------------------------------------

Function TRowGrid.CellRect(ARow,ACol:Integer): TRect;
Begin
  Result.Top:=(ARow-fTopRow+1)*fRowHeight;
  Result.Bottom:=Result.Top+fRowHeight-1; //-1 pour ligne de grille
  If ACol=0
    then Result.Left:=0
    else Result.Left:=fPColsInfos^[ACol-1].Right;
  Result.Right:=fPColsInfos^[ACol].Right;
End;

Procedure TRowGrid.WMEraseBkGnd(var Msg:TMessage);
Begin
  Msg.Result:=1; //Indique  win qu'effacement a t fait
End;

{Surcharge de TCustomControl.Paint pour tout dessiner}
Procedure TRowGrid.Paint;
Begin
  DrawHeaderRow;
  PaintAllRows;
End;

Procedure TRowGrid.PaintAllRows;
Begin
  PaintZone(fTopRow,MaxLongInt);
End;

Procedure TRowGrid.PaintRow(Index:Integer);
Begin
  PaintZone(Index,Index);
End;

Procedure TRowGrid.PaintCell(ARow,ACol:Integer); //Raffichage d'une seule cellule
Var Rect:TRect;
Begin
  If (ARow<fTopRow) or (ARow>Min(fTopRow+fVisibleRowCount,fRowCount-1)) then Exit;
  Rect.Top:=(ARow-fTopRow+1)*fRowHeight;
  Rect.Bottom:=Rect.Top+fRowHeight-1;
  If ACol=0
    then Rect.Left:=0
    else Rect.Left:=fPColsInfos^[ACol-1].Right;
  Rect.Right:=fPColsInfos^[ACol].Right - 1; //1 pour GrigLineWidth

  //Trace la cellule
  DrawCell(ARow,ACol, Rect);
End;

//---------------------- DESSIN ------------------------

Procedure TRowGrid.DrawHeaderRow;
Var
  Col :Integer;
  DC  :THandle;   //Pour viter appels multiples de getDC
  Rect:TRect;     //Rectangle de la cellule
  Text:String;    //Chaine  tracer
  L,T :Integer;   //Pour trac du triangle de tri
Begin
  Canvas.Font.Assign(Font);
  Canvas.Brush.Color:=fColors[ciHeaderRow];
  Canvas.Brush.Style:=bsSolid;
  DC := Canvas.Handle; {Evite appels multiples}

  Rect.Top:=0;
  Rect.Bottom:=fRowHeight;
  Rect.Left:=0;
  If fColCount=0 then
  Begin
    Rect.Right:=ClientWidth;
    DrawEdge(DC, Rect, EDGE_RAISED, BF_RECT or BF_SOFT or BF_MIDDLE);
  End
  else
  Begin
    For Col:=0 to fColCount-1 do
    Begin
      Rect.Right:=fPColsInfos^[Col].Right;

      //Dessine bordure de style bouton
      DrawEdge(DC, Rect, EDGE_RAISED, BF_RECT or BF_SOFT or BF_MIDDLE);

      //Dessine triangle indicateur de tri
      If fSortEnable and (fSortCol=Col) then
      Begin
        L:=Rect.Left+3;
        T:=(fRowHeight-7) div 2;
        If fSortDirection = sdAscending then
        Begin
          Canvas.Pen.Color:=clWindow;
          Canvas.PolyLine([Point(L+6,T), Point(L,T), Point(L+6,T+6)]);
          Canvas.Pen.Color:=clBtnShadow;
          Canvas.PolyLine([Point(L+7,T), Point(L+4,T+6)]);
        End
        else
        Begin
          Canvas.Pen.Color:=clWindow;
          Canvas.PolyLine([Point(L,T+5), Point(L+3,T)]);
          Canvas.Pen.Color:=clBtnShadow;
          Canvas.PolyLine([Point(L,T+6), Point(L+7,T+6), Point(L+4,T)]);
        End;
      End;

      //Dessine le texte
      If fSortEnable
        then Inc(Rect.Left,12) //Rserve place pour marque de tri
        else Inc(Rect.Left,2); //Texte  gauche
      Dec(Rect.Right,2); //Empche criture sur cadre
      Text:=fPColsInfos^[Col].Name;
      DrawText(DC, PChar(Text), Length(Text), Rect, DT_LEFT or {DT_BOTTOM or} DT_VCENTER or
                        DT_END_ELLIPSIS or DT_NOPREFIX or DT_SINGLELINE);
      Rect.Left:=Rect.Right+2;
    End;
  End;
End;


procedure TRowGrid.DrawCell(ARow,ACol:Integer; ARect:TRect);
Const
  BmPlace = 12; //Place for up down bitmap
var
  xOffset, yOffset: Integer;
  Infos: TDrawCellInfo;

  Width: Integer;
  Sauve:TRowPtrSave;
begin
  //Prpare paramtres par dfaut
  Canvas.Font.Assign(Self.Font);
  Infos.Row:=ARow;
  Infos.Col:=ACol;
  Infos.Font := Canvas.Font;
  Infos.Brush := Canvas.Brush;
  Infos.AlignmentHorz := taLeftJustify;
  Infos.AlignmentVert := taMiddle; //taBottomJustify;
  If ARow<>fRow then
    If (rfSel and fPTabRows^[ARow]^.Flags)<>0 then
      Infos.Brush.Color:=fColors[ciSelected]
    else
      If Odd(ARow)
        then Infos.Brush.Color:=fColors[ciOddRow] //Lignes qui n'ont pas le focus
        else Infos.Brush.Color:=fColors[ciEvenRow]
  else
    If ACol<>fCol then
      Infos.Brush.Color:=fColors[ciFocusedRow]
    else
      Infos.Brush.Color:=fColors[ciFocusedCell];

  //Demande modifications spcifiques
  SaveRowPtr(Sauve);
  SelectRow(ARow);

  Infos.Text:=fRowPtr^.Cells[ACol].Str;
  If Assigned(fOnGetCellFormat) then
    fOnGetCellFormat(Self, Infos);

  //Calcule position horizontale et verticale
  Width:=ARect.Right-ARect.Left;
  case Infos.AlignmentHorz of
    taRightJustify: xOffset := Width-2-Canvas.TextWidth(Infos.Text);
    taCenter: xOffset := (Width-Canvas.TextWidth(Infos.Text)) div 2;
    else
      xOffset:=2;
  end;
  case Infos.AlignmentVert of
    taBottomJustify: yOffset:=(ARect.Bottom-ARect.Top)-Canvas.TextHeight(Infos.Text);
    taMiddle: yOffset:=(ARect.Bottom-ARect.Top-Canvas.TextHeight(Infos.Text)) div 2;
    else
      yOffset:=2;
  end;

  //Dessine le texte
  Canvas.TextRect(ARect, ARect.Left+xOffset, ARect.Top+yOffset, Infos.Text);

  If Focused and (ARow=fRow) and (ACol=fCol) then
    Canvas.DrawFocusRect(ARect);
  LoadRowPtr(Sauve);
end;


{Redessine les lignes StartRow  EndRow inclues
 ne redessine jamais les enttes de colonnes}
Procedure TRowGrid.PaintZone(StartRow,EndRow:Integer);
Var
    Col,Row : Integer; //Coordonnes cellule  repeindre
    LastRow : Integer; //Numro dernire ligne  afficher
    Rect: TRect;       //Rectangle de la cellule en cours
Begin
  If StartRow<fTopRow then StartRow:=fTopRow;
  If (StartRow>EndRow) then Exit; {Rien  tracer}

//  Canvas.Font.Assign(Self.Font);
  Canvas.Pen.Color := clSilver;  //Couleur des lignes de grille

  Rect.Top:=(StartRow-fTopRow+1)*fRowHeight;
  LastRow:=Min(Min(EndRow, fTopRow+fVisibleRowCount),fRowCount-1);
  For Row:=Max(StartRow,fTopRow) to LastRow do
  Begin
    //Trace range de cellules
    Rect.Left:=0;
    Rect.Bottom:=Rect.Top+fRowHeight-1;
    For Col:=0 to fColCount-1 do
    Begin
      Rect.Right:=fPColsInfos^[Col].Right - 1; //1 pour GrigLineWidth
      //Trace ligne verticale de sparation
      Canvas.PolyLine([Point(Rect.Right,Rect.Top), Point(Rect.Right,Rect.Bottom)]);
      //Trace la cellule
      DrawCell(Row,Col, Rect);
      Rect.Left:=Rect.Right+1; //+1 pour GridLineWidth
    End;
    //Trace ligne de sparation basse
    Canvas.PolyLine([Point(0,Rect.Bottom), Point(ClientWidth,Rect.Bottom)]);
    Inc(Rect.Top,fRowHeight);
  End;

  If LastRow<EndRow then
  Begin {Vide bas de page innocup par le texte}
    Rect.Bottom:=ClientHeight;
    Rect.Left :=0;
    Rect.Right:=ClientWidth;
    Canvas.Brush.Color:=clWindow;
    Canvas.FillRect(Rect);
  End;
End;

//--------------------------------------------------------------------------
// Gestion des colonnes
//--------------------------------------------------------------------------
Procedure TRowGrid.SetColCountNoPaint(NewCount:Integer);
Var
  Col,Row:Integer;
  NewRowSize:Integer;
  NewColPercent:Single;
  Coef:Single;
  TailleInit:Integer; //Taille  initialiser si ajout de colonnes
Begin
  If (NewCount<>fColCount) and (NewCount>0) then
  Begin
    fColCountChange:=True;
    If fRowCount>0 then
    Begin //met a jout les lignes dj alloues
      //Sup de colonnes : libre les chaines qui vont tre supprimes
      If NewCount<fColCount then
        For Row:=0 to fRowCount-1 do
          For Col:=NewCount to fColCount-1 do
            fPTabRows^[Row]^.Cells[Col].Str:='';

      //Ralloue le contenu des lignes
      NewRowSize:=SizeOf(TRowFlags)+NewCount*SizeOf(TCell);
      For Row:=0 to fRowCount-1 do
        ReallocMem(fPTabRows^[Row], NewRowSize);

      //Met  0 donnes nouvelles colonnes
      If NewCount>fColCount then
      Begin
        TailleInit:=(NewCount-fColCount)*SizeOf(TCell);
        For Row:=0 to fRowCount-1 do
          FillChar(fPTabRows^[Row]^.Cells[fColCount],TailleInit,0);
      End;
    End;

    //Ralloue et init place pour Infos colonnes
    //Ne ralloue pas si suppression de colonnes, la place rcupre
    //est insignifiante par rapport au temps perdu  rallouer
    ReallocMem(fPColsInfos, NewCount*SizeOf(TColInfos));
    Coef:=fColCount/NewCount;
    If NewCount>fColCOunt then
    Begin //Ajoute des colonnes
      For Col:=0 to fColCount-1 do //Ajuste largeurs colonnes existantes
        With fPColsInfos[Col] do
          Percent:=Percent*Coef;

      NewColPercent:=1/NewCount;   //Ajuste nouvelles colonnes
      For Col:=fColCount to NewCount-1 do
        With fPColsInfos[Col] do
        Begin
          Percent:=NewColPercent;
          Pointer(Name):=Nil; //Empche chaines fantmes
        End;
    End;
    fColCount:=NewCount;
  End;
End;

Procedure TRowGrid.SetColCountPaint(NewCount:Integer);
Begin
  SetColCountNoPaint(NewCount);
  UpdateColsWidths;
  SignaleModif;
  Paint; //Repeint tout y compris enttes
End;

Procedure TRowGrid.UpdateColsWidths;
Var
  I:Integer;
  W:Integer; //La colonne, total
  Wdt   :Integer;
  ColMax:Integer;
  WdtMax:Integer;
Begin
  WdtMax:=-1;
  ColMax:=-1;
  W:=0;
  For I:=0 to fColCount-1 do
  Begin
    Wdt:=Round(ClientWidth*fPColsInfos^[I].Percent); //Nouvelle largeur
    Inc(W,Wdt);
    fPColsInfos^[I].Right:=W;

    If Wdt>WdtMax then  //Mmorise colone de largeur max
    Begin
      WdtMax:=Wdt;
      ColMax:=I;
    End;
  End;

  If ColMax>=0 then
  Begin
    W:=ClientWidth-W; //Reporte erreurs de calcul sur colonne la plus large
    For I:=ColMax to fColCount-1 do
      Inc(fPColsInfos^[I].Right, W);
  End;
End;

Function TRowGrid.GetColWidth(Index:Integer):Integer;
Begin
  If Index>=fColCount then
    Result:=-1
  else
  Begin
    Result:= fPColsInfos^[Index].Right;
    If Index>0 then
      Dec(Result,fPColsInfos^[Index-1].Right);
  End;
End;

Procedure TRowGrid.DefColWidths(Const Widths:Array of Integer);
Var
  I:Integer;
  WidthTot:Integer; //Largeur totale donne dans Widths
  Percent:Single;
Begin
  If High(Widths)+1 > fColCount then  //Cre colonnes manquantes si besoin
    SetColCountPaint(High(Widths)+1);

  WidthTot:=Widths[0]; //Calcule largeur totale fournie
  For I:=1 to High(Widths) do
  Begin
    If Widths[I]>2
      then Inc(WidthTot,Widths[I])
      else Inc(WidthTot,10);
  End;

  For I:=0 to High(Widths) do //Calcule largeur en % et largeur relle
  Begin
    If Widths[I]>2
      then Percent:=Widths[I]/WidthTot
      else Percent:=10/WidthTot;
    fPColsInfos^[I].Percent:=Percent;
  End;
  UpdateColsWidths;
  Paint;
End;

Function TRowGrid.GetColName(Index:Integer):String;
Begin
  Result:=fPColsInfos^[Index].Name;
End;
Procedure TRowGrid.SetColName(Index:Integer; Const Name:String);
Begin
  fPColsInfos^[Index].Name:=Name;
  DrawHeaderRow;
End;

Procedure TRowGrid.DefColNames(Const Names:Array of String);
Var
  I:Integer;
  ColCountChange:Boolean;
Begin
  ColCountChange := High(Names)+1<>fColCount;
  If High(Names)+1 > fColCount then
    SetColCountPaint(High(Names)+1);

  For I:=0 to High(Names) do
    fPColsinfos^[I].Name:=Names[I];

  If ColCountChange
    then Paint //Repeint tout, y compris zone entte
    else DrawHeaderRow; //repeint seulement enttes
End;

Procedure TRowGrid.SetColNames(Names:TStrings);
Begin
  fColNames.Assign(Names);
End;

Procedure TRowGrid.ColDefsChange;
Begin
  If fColCountChange then
  Begin
    fColCountChange:=False;
    UpdateColsWidths;
    Paint;
  End
  else
    DrawHeaderRow;
End;

//--------------------------------------------------------------------------
// Filtre
//--------------------------------------------------------------------------

//Slectionne et n'afiche plus que les lignes dont la colonne correspond au
//critre fourni
Procedure TRowGrid.SetFilter(NumCol:Integer);
Var
  I:Integer;
  Sauve:TRowPtrSave;
Begin
  If NumCol>=0 then
  Begin //Mise en place du filtre
    If not Assigned(OnFilter) then Exit;

    SaveRowPtr(Sauve);
    fFilterList:=TList.Create;
    For I:=0 to fRowCount-2 do
    Begin
      SelectRow(I);
      If OnFilter(Self, I,NumCol) then
        fFilterList.Add(fRowPtr);
    End;
    LoadRowPtr(Sauve);
  End
  else
  Begin //Retrait du filtre
    SaveRowPtr(Sauve);

    LoadRowPtr(Sauve);
  End;
End;

//--------------------------------------------------------------------------
// Gestion des donnes
//--------------------------------------------------------------------------

Procedure TRowGrid.BeginUpdate;
Begin
  Inc(fUpdateCount);
End;

Procedure TRowGrid.EndUpdate;
Begin
  If fUpdateCount>0 then
  Begin
    Dec(fUpdateCount);
    If fUpdateCount=0 then
      PaintZone(0,MaxLongInt); //Seulement zone donnes
  End;
End;
{
Function TRowGrid.GetRowPtr(Index:Integer):PRow;
Begin
  If Assigned(fFilterList)
    then Result:=fPTabRows^[Integer(fFilterList[Index])]
    else Result:=fPTabRows[Index];
End;
}
Function TRowGrid.GetRowCount:Integer;
Begin
  Result:=fRowCount-1;
End;

Procedure TRowGrid.SetRowCountNoPaint(NewCount:Integer);
Var
  Col,Row:Integer;
  RowSize:Integer;
Begin
  Inc(NewCount);
  If NewCount<>fRowCount then
  Begin
    If NewCount>fRowsAlloc then
    Begin
      Inc(fRowsAlloc,(((NewCount-fRowsAlloc) div 1000) +1)*1000);
      ReallocMem(fPTabRows,fRowsAlloc*SizeOf(PRow));
      fRowPtr:=fPTabRows^[fRowPtrNum];
    End;

    RowSize:=SizeOf(TRowFlags)+fColCount*SizeOf(TCell);
    If NewCount>fRowCount then
    Begin
      While fRowCount<NewCount do //Ajoute des lignes
      Begin
        fRowPtr:=AllocMem(RowSize); //Alloue et met  0
        fPTabRows^[fRowCount]:=fRowPtr; //Prpare en mme temps fRowPtr
        Inc(fRowCount);
        If Assigned(fOnInitNewRow) then  //Effectue vnement associ
          fOnInitNewRow(Self, fRowCount-1);
      End;
    End
    else
    Begin
      //Dtruit lignes inutiles
      For Row:=NewCount to fRowCount-2 do
      Begin
        For Col:=0 to fColCount-1 do
          fPTabRows^[Row]^.Cells[Col].Str:=''; //Libre mmoires occupe par chaines
        FreeMem(fPTabRows^[Row]); //Libre pointeurs et donnes
      End;
      //Transfre dernire ligne (vierge)
      fPTabRows^[NewCount-1]:=fPTabRows^[fRowCount-1];
      fRowCount:=NewCount;

      //Recadre zone visible
      If fRow>=NewCount then
        fRow:=NewCount-1;
      If fTopRow>=NewCount-fVisibleRowCount then
        SetTopRow(Max(0,NewCount-fVisibleRowCount));
    End;
  End;
End;

Procedure TRowGrid.SetRowCountPaint(NewCount:Integer);
Var DebZone:Integer;
Begin
  If NewCount<>fRowCount-1 then
  Begin
    DebZone:=Min(fRowCount,NewCount);
    SetRowCountNoPaint(NewCount);
    PaintZone(DebZone,Max(fRowCount,NewCount));
    MajScrollBar;
    SignaleModif;
  End;
End;

Function TRowGrid.IsRowEmpty(Row:Integer):Boolean;
Var
  Col:Integer;
  Sauve:TRowPtrSave;
Begin
  If Assigned(fOnRowEmptyCheck) then
  Begin
    SaveRowPtr(Sauve);
    SelectRow(Row);
    Result:=fOnRowEmptyCheck(Self, Row);
    LoadRowPtr(Sauve);
  End
  else
  Begin
    Result:=True; //Juste pour viter warning, ColCount=Constante<>0
    For Col:=0 to fColCount-1 do
    Begin
      Result:=fPTabRows^[Row]^.Cells[Col].Str='';
      If Not Result then
        Break;
    End;
  End;
End;

Procedure TRowGrid.AddRowVide;
Begin
  If (fRow=fRowCount-1) and Not IsRowEmpty(fRow) then
    SetRowCountPaint(fRowCount);
End;

Procedure TRowGrid.InsertRowNoPaint(Index:Integer; PtrNewRow:PRow);
Begin
  If Index>fRowCount then         //Ne peut insrer au dela de la dernire !
    Index:=fRowCount;

  SetRowCountNoPaint(fRowCount);  //Ajoute une ligne en fin de table et la vide

  If Index<=fRow then
    Inc(fRow);

  If Index<fRowCount-1 then
    MoveRowNoPaint(fRowCount-1, Index);  //Dplace la ligne  l'emplacement souhait

  If Assigned(PtrNewRow) then
  Begin
    FreeMem(fPTabRows^[Index]);
    fPTabRows^[Index]:=PtrNewRow;
  End;

  If Assigned(fOnInsertRow) then  //Effectue vnement associ
    fOnInsertRow(Self, Index);

  If Index<=fRow then
    SetActiveRow(fRow-1);
End;

Procedure TRowGrid.InsertRow(Index:Integer);
Begin
  InsertRowNoPaint(Index,NIL); //Ajoute Ligne vide SOUS le curseur
  SignaleModif;
  PaintZone(Index,MaxLongInt); //Repeint toute la zone visible qui change
End;

Procedure TRowGrid.DeleteRow(Index:Integer);
Begin
  If Index>=fRowCount-1 then Exit; //Ne peut pas supprimer dernire ligne

  If IsRowEmpty(fRow) or (Not Assigned(fOnDeleteRowQuery)) or
     fOnDeleteRowQuery(Self, fRow) then
  Begin
    If Assigned(fOnDeleteRow) then
      fOnDeleteRow(Self, Index);

    If Index<fRow then
      Dec(fRow);

    If Index<fRowCount-2 then
      MoveRowNoPaint(Index,fRowCount-2);

    SetRowCountNoPaint(fRowCount-2);
    SignaleModif;
    PaintZone(Index,MaxLongInt);//Repeint toute la zone visible qui change
  End;
End;

Procedure TRowGrid.MoveRowNoPaint(FromIndex,ToIndex:Integer);
Var
  PRowTemp:PRow;
Begin
  If FromIndex<>ToIndex then
  Begin
    PRowTemp:=fPTabRows^[FromIndex]; //Sauve ligne Source
    If ToIndex<FromIndex
      then Move(fPTabRows^[ToIndex], fPTabRows^[ToIndex+1], (FromIndex-ToIndex)*SizeOf(PRow))
      else Move(fpTabRows^[FromIndex+1], fPTabRows^[FromIndex], (ToIndex-FromIndex)*SizeOf(PRow));
    fPTabRows^[ToIndex]:=PRowTemp;
  End;
End;

Procedure TRowGrid.MoveRow(FromIndex,ToIndex:Integer);
Begin
  MoveRowNoPaint(FromIndex, ToIndex);
  PaintZone(Min(FromIndex,ToIndex), Max(FromIndex,ToIndex));
End;

Procedure TRowGrid.SignaleModif;
Var
  Sauve:TRowPtrSave;
Begin
  If csLoading in ComponentState then Exit;

  SaveRowPtr(Sauve);
  Try
    If Not fModified then
    Begin
      fModified:=True;
      If Assigned(fOnModif) then fOnModif(Self);
    End;
    If Assigned(fOnChange) then fOnChange(Self);
  Finally
    LoadRowPtr(Sauve);
  End;
End;

{Accs par ligne}
Procedure TRowGrid.SelectRow(RowNum:Integer);
Begin
  fRowPtrNum:=RowNum; //Pour pouvoir redessiner la ligne slectionne
  fRowPtr:=fPTabRows^[RowNum];
End;

Procedure TRowGrid.SaveRowPtr(Var Place:TRowPtrSave);
Begin
  Place.Ptr:=fRowPtr;
  Place.Idx:=fRowPtrNum;
End;

Procedure TRowGrid.LoadRowPtr(Const Place:TRowPtrSave);
Begin
  fRowPtr:=Place.Ptr;
  fRowPtrNum:=Place.Idx;
End;

Function TRowGrid.GetRowFlags:TRowFlags;
Begin
  Result:=fRowPtr^.Flags;
End;

Procedure TRowGrid.SetRowFlags(NewFlags:TRowFlags);
Begin
  With fRowPtr^ do
  If Flags<>NewFlags then
  Begin
    If (Flags and rfSel =0) and (NewFlags and rfSel <>0) then
      Inc(fSelCount);
    If (Flags and rfSel <> 0) and (NewFlags and rfSel =0) then
      Dec(fSelCount);

    fRowPtr^.Flags:=NewFlags;
    PaintRow(fRowPtrNum);
  End;
End;

Function TRowGrid.GetRowCellStr(Col:Integer):String;
Begin
  Result:=fRowPtr^.Cells[Col].Str;
End;
Procedure TRowGrid.SetRowCellStr(Col:Integer; S:String);
Begin
  With fRowPtr^.Cells[Col] do
  Begin
    If Str<>S then
    Begin
      Str:=S;
      PaintCell(fRowPtrNum,Col);
      AddRowVide; //Ajoute une ligne si dernire n'est plus vide
    //SignaleModif; seulement pour dition par utilisateur
    End;
  End;
End;

Function TRowGrid.GetRowCellDat(Col:Integer):Integer;
Begin
  Result:=fRowPtr^.Cells[Col].Dat;
End;
Procedure TRowGrid.SetRowCellDat(Col,Data:Integer);
Begin
  With fRowPtr^.Cells[Col] do
  Begin
    If Dat<>Data then
    Begin
      Dat:=Data;
      AddRowVide; //Ajoute une ligne si dernire n'est plus vide
    //SignaleModif; seulement pour dition par utilisateur
    End;
  End;
End;

//--------------------------------------------------------------------------
//  Focus et Gestion de la cellule et ligne slectionne
//--------------------------------------------------------------------------

Procedure TRowGrid.WMSetFocus(var Msg: TWMSetFocus);
begin
  inherited;
  PaintCell(fRow,fCol);
end;

Procedure TRowGrid.WMKillFocus(Var Msg: TWMKillFocus);
begin
  inherited;
  PaintCell(fRow,fCol);
end;

Procedure TRowGrid.SetActiveCol(NewCol:Integer);
Var
  ColPrec:Integer;
Begin
  If NewCol=fCol then Exit;

  ColPrec:=fCol; //Memorise Col active prcdente
  fCol:=NewCol;
  PaintCell(fRow,ColPrec);  //Efface marque focus sur ancienne ligne
  PaintCell(fRow,fCol);     //Trace marque focus sur nouvelle ligne
  If Assigned(fOnColChange) then
    fOnColChange(Self, ColPrec);
End;

Procedure TRowGrid.SetActiveRow(NewRow:Integer);
Var
  RowPrec:Integer;
Begin
  If NewRow=fRow then Exit;

  RowPrec:=fRow; //Memorise row active prcdente
  fRow:=NewRow;
  PaintRow(RowPrec); //Efface marque focus sur ancienne ligne
  PaintRow(fRow);    //Trace marque focus sur nouvelle ligne
  If Assigned(fOnRowChange) then
    fOnRowChange(Self, RowPrec);
End;

Function TRowGrid.CanSelect(ARow,ACol:Integer):Boolean;
Begin
  Result:=True;
  If Assigned(fOnSelectCell) then
    fOnSelectCell(Self, ARow,ACol, Result);
End;

//--------------------------------------------------------------------------
// Gestion de la souris
//--------------------------------------------------------------------------

//Autorise actions souris dans zones boutons pendant conception
procedure TRowGrid.CMDesignHitTest(var Msg: TCMDesignHitTest);
begin
  With Msg.Pos do
    Msg.Result := Longint(Boolean((X>=0) and (X<Width) and (Y>=0) and (Y<fRowHeight)));
End;

Procedure TRowGrid.MouseToCell(X,Y:Integer; var ARow,ACol:Integer);
Begin
  If Y<fRowHeight then
    ARow:=-1
  else
    ARow:=(Y div fRowHeight) + fTopRow-1;

  ACol:=0;
  While fPColsInfos^[ACol].Right<=X do
    Inc(ACol);
End;

Procedure TRowGrid.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
var
  Col,Row: Integer;
begin
  If Not Focused then //Redonne focus  contrle si perdu
    SetFocus;

  MouseToCell(X,Y, Row,Col); //Calcule Col et Row

  If Button=mbLeft then
  Begin
    If Row<0 then //Click dans barre de titre
    Begin
      If fColSizing then
      Begin //Dbut de redimentionnement de colonne
        fPenMode:=Canvas.Pen.Mode; //Sauve mode courant
        Canvas.Pen.Mode:=pmNot;    //Met en mode trace/efface
        Canvas.Polyline([Point(X,0),Point(X,ClientHeight)]);
        fColSizeLine:=X; //Mmo ligne courante

        fCol:=Col;
        If X<fPColsInfos^[fCol].Right-4 then
          Dec(fCol);
      End
      else
        fMouseDownCol := Col; //Click dans titres de colonnes
    End
    else //Click dans zone donnes
      If Row<fRowCount then
      Begin //Click dans ligne existante
        fMouseDownCol:=-1; //Annule click dans
        If ssCtrl in Shift then
        Begin
          If Row<fRowCount-1 then
          With fPTabRows^[Row]^ do
          Begin
            Flags:=Flags xor rfSel; //permute flag slection.
            If Flags and rfSel=0
              then Dec(fSelCount)
              else Inc(fSelCount);
            PaintRow(Row);
          End
        End
        else
          If (Row=fRow) and (Col=fCol) then
            EnterEdit
          else
            If CanSelect(Row,Col) then
              If fRow<>Row then
              Begin //Ligne change, peut-tre colonne aussi
                fCol:=Col;
                SetActiveRow(Row);
              end
              else
              Begin //Colonne change, mme ligne
                fCol:=Col;
                PaintRow(fRow);
              End;
      End;
  End;

  Inherited; {Appelle OnMouseDown si dfini}
End;

Procedure TRowGrid.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
var
  Col,Row : Integer;
  X0,X1,X2:Integer;
begin
  If Button=mbLeft then
    If fColSizing then
    Begin
      Canvas.Polyline([Point(fColSizeLine,0),Point(fColSizeLine,ClientHeight)]);
      Canvas.Pen.Mode:=fPenMode; //Remet en mode par dfaut
      If fCol=0 then X0:=0 else X0:=fPColsInfos^[fCol-1].Right;
      X1:=fPColsInfos^[fCol].Right;
      X2:=fPColsInfos^[fCol+1].Right;
      fPColsInfos^[fCol  ].Percent:=fPColsInfos^[fCol  ].Percent*(fColSizeLine-X0)/(X1-X0);
      fPColsInfos^[fCol+1].Percent:=fPColsInfos^[fCol+1].Percent*(X2-fColSizeLine)/(X2-X1);
      UpdateColsWidths;
      Paint; //Repeint tout y compris enttes de colonnes
    End
    else
    Begin
      MouseToCell(X,Y,Row,Col);
      If (Row=-1) and (Col=fMouseDownCol) then
      Begin
        If (fSortCol=Col) and (fSortDirection=sdAscending)
          then fSortDirection := sdDescending
          else fSortDirection := sdAscending;

        SortColumn(Col);
        DrawHeaderRow;
      End;
    End;
  Inherited;
End;

Procedure TRowGrid.MouseMove(Shift:TShiftState; X,Y:Integer);
Var
  I,N:Integer;
Begin
  If ssLeft in Shift then
  Begin
    If fColSizing then
    Begin
      Canvas.Polyline([Point(fColSizeLine,0),Point(fColSizeLine,ClientHeight)]);
      If fCol=0 then
      Begin
        If X<2 then X:=2;
      End
      else
        If X<fPColsInfos^[fCol-1].Right+14 then
          X:=fPColsInfos^[fCol-1].Right+14;
      If X>fPColsInfos^[fCol+1].Right-14 then
        X:=fPColsInfos^[fCol+1].Right-14;

      Canvas.Polyline([Point(X,0),Point(X,ClientHeight)]);
      fColSizeLine:=X;
    End;
  End
  else
  Begin
    fColSizing:=False;
    If Y<fRowHeight then
    Begin //Dans zone Boutons
      For I:=0 to fColCount-2 do
      Begin
        N:=fPColsInfos^[I].Right;
        fColSizing:=(X>N-4) and (X<N+2);
        If fColSizing then Break;
      End;
    End;

    If fColSizing
      then Cursor:=crHSplit
      else Cursor:=crArrow;
  End;
  Inherited; {Appelle OnMouseMove si dfini}
End;

//--------------------------------------------------------------------------
// Gestion du clavier
//--------------------------------------------------------------------------

Procedure TRowGrid.WMGetDlgCode(var Msg:TWMGetDlgCode);
Begin
  Msg.Result:=DLGC_WANTARROWS   {TRowGrid traite touches curseur}
           or DLGC_WANTCHARS    {TRowGrid traite caractres}
           or DLGC_WANTTAB      {TRowGrid traite tabulations}
           or DLGC_WANTALLKEYS;
End;

Procedure TRowGrid.KeyDown(var Key:Word; Shift:TShiftState);

  // Procedure commune aux dplacements verticaux du curseur
  Procedure MoveActiveRowTo(N:Integer);
  Begin
    If N<0 then N:=0;
    If N>=fRowCount then N:=fRowCount-1;
    If (N<fTopRow) or (N-fVisibleRowCount>=fTopRow) then
    Begin
      If N<fTopRow
        then fTopRow:=N
        else fTopRow:=N-fVisibleRowCount+1;
      Paint;
      MajScrollBar;
    End;
    SetActiveRow(N);
  End;

Begin
  Inherited;  {Appelle OnKeyDown si dfini}
  If Key=0 then Exit; {Touche intgralement traite par OnKeyDown}

     If (ssCtrl in Shift) and (Key<=255) then
    Case Key of
      Ord('C'): //Ctrl-C
        CopyToClipboard;

      Ord('V'): //Ctrl-V
        PasteFromClipboard;

      Ord('X'): //Ctrl-X
        CutToClipboard;

      Ord('Y'): {Supprime la ligne sous le curseur}
        If Not ReadOnly then
          DeleteRow(fRow);

      VK_UP:{Ctrl-Haut : Scroll 1 ligne plus haut}
      If fTopRow>0 then
      Begin
        Dec(fTopRow);
        MajScrollBar;
        PaintAllRows;
      End;

      VK_DOWN:{Ctrl-Bas : Scroll 1 ligne plus bas}
      If fTopRow+fVisibleRowCount<fRowCount then
      Begin
        Inc(fTopRow);
        MajScrollBar;
        PaintAllRows;
      End;

      VK_PRIOR, {Ctrl-PageHaut}
      VK_HOME:  {Ctrl-Home : Dbut de la table}
        MoveActiveRowTo(0);

      VK_NEXT, {Ctrl-PageBas}
      VK_END:  {Ctrl-Fin : Fin de la table}
        MoveActiveRowTo(fRowCount-1);
    end {case Ctrl-key}
  else {Pas Ctrl-xxx}
  Begin
    Case Key of
      VK_LEFT: {1 cellule  gauche}
      Begin
        Repeat
          If fCol>0
            then Dec(fCol)
            else fCol:=fColCount-1;
        Until CanSelect(fRow,fCol);
        PaintRow(fRow);
      End;

      VK_RIGHT: {1 car  droite}
      Begin
        Repeat
        If fCol<fColCount-1
          then Inc(fCol)
          else fCol:=0;
        Until CanSelect(fRow,fCol);
        PaintRow(fRow);
      End;

      VK_UP: {1 ligne plus haut}
        MoveActiveRowTo(fRow-1);

      VK_DOWN: {1 ligne plus bas}
        MoveActiveRowTo(fRow+1);

      VK_PRIOR: {1 page plus haut}
        If fRow=fTopRow
          then MoveActiveRowTo(fRow-fVisibleRowCount)
          else MoveActiveRowTo(fTopRow);

      VK_NEXT: {1 page plus bas}
        If fRow=fTopRow+fVisibleRowCount-1
          then MoveActiveRowTo(fRow+fVisibleRowCount)
          else MoveActiveRowTo(fTopRow+fVisibleRowCount-1);

      VK_HOME: {Dbut de ligne}
      Begin
        fCol:=0;
        While Not CanSelect(fRow,fCol) do
          Inc(fCol);
        PaintRow(fRow);
      End;

      VK_END: {Fin de ligne}
      Begin
        fCol:=fColCount-1;
        While Not CanSelect(fRow,fCol) do
          Dec(fCol);
        PaintRow(fRow);
      End;

      VK_INSERT: {Insre une ligne vierge}
        If Not ReadOnly then
          InsertRow(fRow);

      VK_DELETE: {Vide cellule ou supprime ligne si vide }
        If not ReadOnly and (RowCount>1) then
        Begin
          With fPTabRows^[fRow]^.Cells[fCol] do
          If Str<>'' then
          Begin
            EnterEdit;
            If fEdit is TEdit then
              TEdit(fEdit).Text:='';
          End
          else
            DeleteRow(fRow);
        End;

      VK_BACK: //entre diteur par la droite
      Begin
        EnterEdit;
        If fEdit is TEdit then
          TEdit(fEdit).SelStart:=Length(TEdit(fEdit).Text);
      End;
    End; {Case Key}
  end; {pas Ctrl}
End;

Procedure TRowGrid.KeyPress(var Key:Char);
Begin
  Inherited;
  If (Key>=' ') and (Key<=#$FF) then
  Begin
    EnterEdit;
    If (fEdit is TEdit) then
      If Key=' ' then
        TEdit(fEdit).SelStart:=0
      else
      Begin
        TEdit(fEdit).Text:=Key;
        TEdit(fEdit).SelStart:=1
      End;
  End;
End;

Function TRowGrid.Editing:Boolean;
Begin
  Result:=Assigned(fEdit) and fEdit.Visible;
End;

Procedure TRowGrid.EnterEdit;
Var Edit:TEditBaseClass;
Begin
  If ReadOnly then Exit;

  If Assigned(fOnCreateEditor) then
  Begin
    Edit:=fOnCreateEditor(Self, fCol);
    If Assigned(Edit) then
    Begin
      If Assigned(fEdit) then
        fEdit.Destroy;
      fEdit:=Edit;
    End;
  End;

  If Not Assigned(fEdit) then
    fEdit:=TRowGridEdit.Create(Self);
  fEdit.Parent:=Self;

  SetEditPlace;
  fEdit.Show;
  fEdit.SetFocus;
End;

Procedure TRowGrid.SetEditplace;
Var
  Rect:TRect;
Begin
  Rect:=CellRect(fRow,fCol);
  If Rect.Top=0 then //Empche diteur de se trouver dans barre boutons
    Rect.Top:=-fRowHeight;
  fEdit.SetBounds(Rect.Left, Rect.Top, Rect.Right-Rect.Left, fRowHeight);
End;

//--------------------------------------------------------------------------
// Tris
//--------------------------------------------------------------------------
function TRowGrid.SortInteger(Const Item1,Item2:TSortElement):Integer;
begin
  If Integer(Item1.Key1)<>Integer(Item2.Key1) then
    If Integer(Item1.Key1)>Integer(Item2.Key1)
      then Result:=1
      else Result:=-1
  else
    If Integer(Item1.Key2)=Integer(Item2.Key2) then
      Result:=0
    else
      If Integer(Item1.Key2)>Integer(Item2.Key2)
        then Result:=1
        else Result:=-1;
end;

function TRowGrid.SortStrings(Const Item1,Item2:TSortElement):Integer;
begin
  Result:=CompareText(PChar(Item1.Key1),PChar(Item2.Key1));
  If Result=0 then
    If Integer(Item1.Key2)=Integer(Item2.Key2) then
      Result:=0
    else
      If Integer(Item1.Key2)>Integer(Item2.Key2)
        then Result:=1
        else Result:=-1;
end;


Procedure TRowGrid.SetSortEnable(Etat:Boolean);
Begin
  If fSortEnable<>Etat then
  Begin
    fSortEnable:=Etat;
    DrawHeaderRow;
  End;
End;

Procedure TRowGrid.SortColumn(ACol:Integer);
Type
  TSortList = array[0..MaxLongInt div SizeOf(TSortElement)-1] of TSortElement;
  PSortList = ^TSortList;
Var
  I : Integer;
  ItemMax:Integer;
  SortList:PSortList;
  UserGetKey:Boolean;
  Dest:Integer;
  CurRowAdr:Pointer;
  CurRowPos:Integer;

  //Variables non rcursives de Tri
  Buf:TSortElement;  //Pointeur vers lment  trier
  Swap:TSortElement; //Buffer pour change

   Procedure Tri(D,F:Integer); //Attention : I ou J peuvent tre ngatifs
   Var
     I,J:Integer;
   Begin
     I:=D;
     J:=F;
     Buf:=SortList^[(D+F) div 2];
     Repeat
       While fOnSortComp(SortList^[I],Buf)<0 do Inc(I);
       While fOnSortComp(SortList^[J],Buf)>0 do Dec(J);
       If I<J then
       Begin
         Swap:=SortList^[I];
         SortList^[I]:=SortList^[J];
         SortList^[J]:=Swap;
       End;
       If I<=J then
       Begin
         Inc(I);
         Dec(J);
       End;
     Until I>J;

     If J-D<F-I then  //Commence par le tri le plus court
     Begin            //pour limiter l'emploi de la pile
       If D<J then Tri(D,J);
       If I<F then Tri(I,F);
     End
     else
     Begin
       If I<F then Tri(I,F);
       If D<J then Tri(D,J);
     End
   End;

Begin
  If not fSortEnable then Exit;

  If Assigned(fOnSortStart) then
    fOnSortStart(Self, ACol);

  If Assigned(fOnSortComp) then
  Begin
    fSortCol:=ACol;
    //Cre liste des cls  trier
    GetMem(SortList,(fRowCount-1)*SizeOf(TSortElement));
    Screen.Cursor:=crHourGlass;
    Try
      //Sauve adresse et position ligne actuelle
      CurRowAdr:=fPTabRows^[fRow];
      CurRowPos:=fRow-fTopRow;

      //Met en place les cls de tri en supprimant les lignes vides
      UserGetKey:=Assigned(fOnSortGetKey);
      I:=0; ItemMax:=fRowCount-2;
      While I<=ItemMax do
        If IsRowEmpty(I) then
        Begin //Ligne vide, la met  la fin pour la supprimer
          MoveRowNoPaint(I,ItemMax);
          Dec(ItemMax);
        End
        else
        Begin
          SortList^[I].Index:=I;
          If UserGetKey then
          Begin
            fRowPtr:=fPTabRows^[I]; //Effectue SelectRow
            fOnSortGetKey(ACol, SortList^[I]);
          End
          else
          Begin
            SortList^[I].Key1:=PChar(fPTabRows[I]^.Cells[ACol].Str);
            SortList^[I].Key2:=Pointer(I);
          End;
          Inc(I);
        End;
      SetRowCountNoPaint(ItemMax+1);

      If ItemMax>=0 then
      Begin
        //Trie la liste
        Tri(0, ItemMax);

        //Remplace les index par les pointeurs vers les lignes
        For I:=0 to ItemMax do
          With SortList^[I] do
            Index:=Integer(fPTabRows[Index]);

        //Reconstruit la table ordonne des pointeurs vers les lignes
        Dest:=1;
        If fSortDirection=sdAscending then
        Begin
          fPTabRows[0]:=Pointer(SortList^[0].Index);
          For I:=1 to ItemMax do
          Begin
            If Assigned(fOnSortInsSep) and
               fOnSortInsSep(SortList^[I-1],SortList^[I]) then
            Begin
              InsertRowNoPaint(Dest,NIL); //Insre ligne vide
              Inc(Dest);
            End;
            fPTabRows[Dest]:=Pointer(SortList^[I].Index);
            Inc(Dest);
          End;
        End
        else
        Begin
          fPTabRows[0]:=Pointer(SortList^[ItemMax].Index);
          For I:=ItemMax-1 downto 0 do
          Begin
            If Assigned(fOnSortInsSep) and
               fOnSortInsSep(SortList^[I+1],SortList^[I]) then
            Begin
              InsertRowNoPaint(Dest,NIL); //Insre ligne vide
              Inc(Dest);
            End;
            fPTabRows[Dest]:=Pointer(SortList^[I].Index);
            Inc(Dest);
          End;
        End;
      End;
      
      //Rcupre index de la ligne courante, tente de la remettre au mme endroit
      For I:=0 to fRowCount-2 do
        If fPTabRows^[I]=CurRowAdr then
        Begin
          fRow:=I;
          Break;
        End;
      SetTopRow(Max(0, Min(fRow-CurRowPos, fRowCount-fVisibleRowCount)));

    Finally
      Screen.Cursor:=crArrow;
      FreeMem(SortList);
    End;
  End;
  If Assigned(fOnSortEnd) then
    fOnSortEnd(Self, ACol);

  Paint;
  SignaleModif;
end;


//--------------------------------------------------------------------------
// Copier/Couper/Coller
//--------------------------------------------------------------------------

Procedure TRowGrid.CopyToClipboard;
Var
  TextTaille:Integer;//Taille de la copie CF_TEXT
  DataTaille:Integer;//Taille de la copie CF_RowGrid
  LinesCount:Integer;//Nombre de lignes  copier
  R,C:Integer;       //Compteur de lignes, colonnes
  DataHandle:THandle;//Handle pour donnes CF_TRowGridRows
  TextHandle:THandle;//Handle pour donnes CF_TEXT
  P:Pointer;         //Pointeur sur mmoire globales
  Procedure PutInt(N:Integer);
  Begin
    Pinteger(P)^:=N;
    Inc(PChar(P),SizeOf(Integer));
  End;
  Procedure PutStr(Const S:String);
  Begin
    P:=StrECopy(P,PChar(S));
    Inc(PChar(P)); //Saute le 0 de fin de chaine
  End;
Begin
  If Editing then
  Begin
    If fEdit is TCustomEdit then
      TCustomEdit(fEdit).CopyToClipboard;
  End
  else
  Begin
    //Calcule tailles ncessaires aux donnes
    TextTaille:=0;
    DataTaille:=SizeOf(Integer)*3; //ClipboardId + fColCount + LinesCount;
    LinesCount:=0;
    For R:=0 to fRowCount-2 do
      With fPTabRows^[R]^ do
      If Flags and rfSel<>0 then
      Begin
        Inc(LinesCount);
        Inc(DataTaille,SizeOf(TRowFlags)); //RowFlags
        Inc(DataTaille,SizeOf(Pointer)*fColCount);  //RowDatas[]
        For C:=0 to fColCount-1 do
          Inc(TextTaille,Length(Cells[C].Str)+1);   //RowTexts[]
      End;
    Inc(DataTaille,TextTaille);
    If TextTaille=0 then Exit; //Aucune ligne slectionne

    //Prpare bloc pour export spcifique TRowGrid
    DataHandle:=GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE,DataTaille);
    If DataHandle<>0 then     //Echec allocaton mmoire
    Begin
      P:=GlobalLock(DataHandle); //Copie donnes dans DataPtr^
      Try
        PutInt(fClipboardId);
        PutInt(fColCount);
        PutInt(LinesCount);
        For R:=0 to fRowCount-2 do
          With fPTabRows^[R]^ do
          If Flags and rfSel<>0 then
          Begin
            PutInt(Flags);
            For C:=0 to fColCount-1 do
            Begin
              PutStr(Cells[C].Str);
              PutInt(Cells[C].Dat);
            End;
          End;
      Finally
        GlobalUnlock(DataHandle);
      End;
    End;

    //Prpare bloc de texte spar par tabulations
    TextHandle:=GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE,TextTaille);
    If TextHandle<>0 then     //Echec allocaton mmoire
    Begin
      P:=GlobalLock(TextHandle); //Copie donnes dans DataPtr^
      Try
        For R:=0 to fRowCount-2 do
          With fPTabRows^[R]^ do
          If Flags and rfSel<>0 then
          Begin
            For C:=0 to fColCount-1 do
            Begin
              P:=StrECopy(P, PChar(Cells[C].Str));
              PChar(P)^:=#9; Inc(PChar(P));
            End;
            (PChar(P)-1)^:=#10;
          End;
        (PChar(P)-1)^:=#0;
      Finally
        GlobalUnlock(TextHandle);
      End;
    End;

    If OpenClipboard(0) then //Envoie donnes dans clipboard
    Try
      EmptyClipboard;
      SetClipboardData(CF_TRowGridRow, DataHandle);
      SetClipboardData(CF_TEXT,TextHandle);
    Finally
      CloseClipboard;
    End;
  End;
End;

Procedure TRowGrid.CutToClipBoard;
Var R:Integer;
Begin
  If Editing then
  Begin
    If fEdit is TCustomEdit then
      TCustomEdit(fEdit).CutToClipboard;
  End
  else
  Begin
    If (fClipboardId=0) or ReadOnly then Exit; //Refuse de couper si pas autoris

    CopyToClipboard;
    For R:=fRowCount-2 downto 0 do
      If fPTabRows^[R]^.Flags and rfSel<>0 then
        DeleteRow(R);
    fSelCount:=0; //Toutes les lignes slectionnes ont t supprimes
  End;
End;

Function TRowGrid.CanPaste:Boolean;
Var
  DataHandle:THandle;//Handle de mmoire globale
  P:PChar;           //Pointeur sur mmoire globales
  Function GetInt:Integer;
  Begin
    Result:=PInt(P)^;
    Inc(P,SizeOf(Integer));
  End;
Begin
  Result:=False;
  If (fClipboardId<>0) and IsClipboardFormatAvailable(CF_TRowGridRow) then
  Begin //Rcupre Id dans clipboard
    If OpenClipboard(0) then
    Try
      DataHandle:=GetClipboardData(CF_TRowGridRow);
      P:=GlobalLock(DataHandle);
      Try
        Result:=(GetInt{ID}=fClipboardId) and (GetInt{cols}<>0);
      Finally
        GlobalUnlock(DataHandle);
      End;
    Finally
      CloseClipboard;
    End;
  End;
End;

Procedure TRowGrid.PasteFromClipboard;
Var
  R,C:Integer;       //Compteur de lignes, colonnes
  DataHandle:THandle;//Handle de mmoire globale
  P:PChar;           //Pointeur sur mmoire globales
  Id:Integer;
  Cols:Integer;
  NewRow:PRow;       //Pointeur vers nouvelle ligne
  RowSize:Integer;   //Taille en mmoire d'une row.
  Where:Integer;     //Ligne d'insersion.

  Function GetInt:Integer;
  Begin
    Result:=PInt(P)^;
    Inc(P,SizeOf(Integer));
  End;
  Function GetStr:String;
  Var L:Integer;
  Begin
    L:=StrLen(P);
    SetString(Result,P,L);
    Inc(P,L+1);
  End;

Begin
  If Editing then
  Begin
    If fEdit is TCustomEdit then
      TCustomEdit(fEdit).PasteFromClipboard;
  End
  else
  Begin
    If (fClipboardId=0) or ReadOnly then Exit; //Refuse de coller si pas autoris

    If OpenClipboard(0) then
    Try
      If IsClipboardFormatAvailable(CF_TRowGridRow) then
      Begin //Rcupre lignes et les insre
        DataHandle:=GetClipboardData(CF_TRowGridRow);
        P:=GlobalLock(DataHandle); //Copie donnes dans DataPtr^
        Try
          Id:=GetInt; Cols:=GetInt;
          If Id=fClipboardId then
          Begin
            RowSize:=SizeOf(TRowFlags)+fColCount*SizeOf(TCell);
            Where:=fRow;
            For R:=0 to GetInt-1 do
            Begin
              NewRow:=AllocMem(RowSize); //Raz et empche chaines fantmes
              NewRow^.Flags:=GetInt and NOT rfSel;
              For C:=0 to Cols-1 do
              Begin
                NewRow^.Cells[C].Str:=GetStr;
                NewRow^.Cells[C].Dat:=GetInt;
              End;
              If Not Assigned(fOnPaste) or fOnPaste(NewRow^) then
              Begin
                InsertRowNoPaint(Where,NewRow);
                Inc(Where);
              End;
            End;
            SetActiveRow(Where-1);
          End;
        Finally
          GlobalUnlock(DataHandle);
        End;
        Paint;
        SignaleModif;
      End;
    Finally
      CloseClipboard;
    End;
  End;
End;

//--------------------------------------------------------------------------
// Sauvegarde / chargement proprits particulires
//--------------------------------------------------------------------------

Procedure TRowGrid.ReadColWidths(Reader:TReader);
Type
  TIntArray=Array[0..1000] of Integer;
  PIntArray=^TIntArray;
Var
  I:Integer;
  PW:PIntArray;     //Pointeur vers table lue
  AWidth:Integer;
  WidthTot:Integer;
Begin
  GetMem(PW, fColCount*SizeOf(Integer));
  With Reader do
  Begin
    ReadListBegin;
    WidthTot:=ReadInteger;
    PW^[0]:=WidthTot;
    For I:=1 to fColCount-1 do
    Begin
      AWidth:=ReadInteger;
      If AWidth<10 then AWidth:=10;
      PW^[I]:=AWidth;
      Inc(WidthTot,AWidth);
    End;
    ReadListEnd;
  End;

  For I:=0 to fColCount-1 do //Calcule largeur en % et largeur relle
    fPColsInfos^[I].Percent:=PW^[I]/WidthTot;
  FreeMem(PW);
  UpdateColsWidths; //Recalcule Right
End;

Procedure TRowGrid.WriteColWidths(Writer:TWriter);
Var I:Integer;
Begin
  With Writer do
  Begin
    WriteListBegin;
    WriteInteger(fPColsInfos^[0].Right);
    For I:=1 to fColCount-1 do
      WriteInteger(fPColsInfos^[I].Right-fPColsInfos^[I-1].Right);
    WriteListEnd;
  End;
End;

Procedure TRowGrid.DefineProperties(Filer:TFiler);
  Function DoColWidths: Boolean;
  Begin
    Result:=fColCount<>0; //Sauve sauf si pas de colonnes
  End;
begin
  Inherited;
  with Filer do
    DefineProperty('ColWidths', ReadColWidths, WriteColWidths, DoColWidths);
end;

Initialization
  CF_TRowGridRow:=RegisterClipboardFormat('TRowGridRowData');
end.
